from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.is_hvac_sys_fluid_loop_purchased_heating import (
    is_hvac_sys_fluid_loop_purchased_heating,
)
from rct229.schema.validate import schema_validate_rmd

TEST_RMD = {
    "id": "test_rmd",
    "buildings": [
        {
            "id": "building_1",
            "building_segments": [
                {
                    "id": "building_segment_1",
                    "heating_ventilating_air_conditioning_systems": [
                        {
                            "id": "hvac_1",
                            "heating_system": {
                                "id": "heating_system_1",
                                "hot_water_loop": "HW_Loop_1",
                            },
                        },
                        {
                            # Case where the preheat system has a wrong hot_water_loop id
                            "id": "hvac_2",
                            "heating_system": {
                                "id": "heating_system_2",
                                "hot_water_loop": "ST_Loop_1",
                            },
                        },
                        {
                            # Case where there is no preheat system
                            "id": "hvac_3",
                            "heating_system": {
                                "id": "heating_system_3",
                                "hot_water_loop": "CHW_Loop_1",
                            },
                        },
                        {
                            # Case where there is no preheat system
                            "id": "hvac_4"
                        },
                    ],
                }
            ],
        }
    ],
    "fluid_loops": [
        {"id": "HW_Loop_1", "type": "HEATING"},
        {"id": "ST_Loop_1", "type": "HEATING"},
        {"id": "CHW_Loop_1", "type": "COOLING"},
    ],
    "external_fluid_sources": [
        {"id": "fluid_loop_1", "loop": "HW_Loop_1", "type": "HOT_WATER"},
        {"id": "fluid_loop_2", "loop": "ST_Loop_1", "type": "STEAM"},
        {"id": "fluid_loop_3", "loop": "CHW_Loop_1", "type": "CHILLED_WATER"},
    ],
    "type": "BASELINE_0",
}

TEST_RMD_FULL = {
    "id": "229_01",
    "ruleset_model_descriptions": [TEST_RMD],
    "data_timestamp": "2024-02-12T09:00Z",
}


def test__TEST_RMD__is_valid():
    schema_validation_result = schema_validate_rmd(TEST_RMD_FULL)
    assert schema_validation_result[
        "passed"
    ], f"Schema error: {schema_validation_result['error']}"


def test__fluid_loop_purchased_heating_hw():
    assert is_hvac_sys_fluid_loop_purchased_heating(TEST_RMD, "hvac_1") == True


def test__fluid_loop_purchased_heating_steam():
    assert is_hvac_sys_fluid_loop_purchased_heating(TEST_RMD, "hvac_2") == True


def test__fluid_loop_purchased_heating_not_external():
    assert is_hvac_sys_fluid_loop_purchased_heating(TEST_RMD, "hvac_3") == False


def test__fluid_loop_purchased_heating_no_hot_water():
    assert is_hvac_sys_fluid_loop_purchased_heating(TEST_RMD, "hvac_4") == False
