from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.are_all_terminal_cool_sources_none_or_null import (
    are_all_terminal_cool_sources_none_or_null,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.are_all_terminal_fans_null import (
    are_all_terminal_fans_null,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.are_all_terminal_heat_sources_none_or_null import (
    are_all_terminal_heat_sources_none_or_null,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.are_all_terminal_supplies_ducted import (
    are_all_terminal_supplies_ducted,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.are_all_terminal_types_CAV import (
    are_all_terminal_types_cav,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.does_each_zone_have_only_one_terminal import (
    does_each_zone_have_only_one_terminal,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.does_hvac_system_serve_single_zone import (
    does_hvac_system_serve_single_zone,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.is_hvac_sys_cooling_type_DX import (
    is_hvac_sys_cooling_type_dx,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.is_hvac_sys_fan_sys_CV import (
    is_hvac_sys_fan_sys_cv,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.is_hvac_sys_fluid_loop_attached_to_boiler import (
    is_hvac_sys_fluid_loop_attached_to_boiler,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.is_hvac_sys_fluid_loop_purchased_heating import (
    is_hvac_sys_fluid_loop_purchased_heating,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_hvac_sub_functions.is_hvac_sys_heating_type_fluid_loop import (
    is_hvac_sys_heating_type_fluid_loop,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.baseline_system_util import (
    HVAC_SYS,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.is_baseline_system_1_a import (
    is_baseline_system_1_a,
)
from rct229.rulesets.ashrae9012019.ruleset_functions.baseline_systems.is_baseline_system_1_c import (
    is_baseline_system_1_c,
)
from rct229.schema.schema_enums import SchemaEnums
from rct229.utils.utility_functions import has_preheat_system

HEATING_SYSTEM = SchemaEnums.schema_enums["HeatingSystemOptions"]


def is_baseline_system_1(rmd_b, hvac_b_id, terminal_unit_id_list, zone_id_list):
    """
    Get either Sys-1, Sys-1a, Sys-1b, Sys-1c, or Not_Sys_1 string output which indicates whether the HVAC system is
    ASHRAE 90.1 2019 Appendix G system 1 (PTAC), system 1a (system 1 with purchased CHW), system 1b (system 1 with
    purchased heating), system 1c (system 1 with purchased CHW and purchased HW).

    Parameters
    ----------
    rmd_b JSON, To evaluate if the hvac system is modeled as either Sys-1, Sys-1a, Sys-1b, Sys-1c, or Not_Sys_1 in the B_RMD.
    hvac_b_id String, The id of the hvac system to evaluate.
    terminal_unit_id_list List<String>, list of terminal unit IDs associated with the HVAC system to be evaluated. These are
                            sent to this function from the master get_baseline_system_types function.
    zone_id_list  List<String>, list of zone IDs associated with the HVAC system to be evaluated. These are sent to this
                        function from the master get_baseline_system_types function.

    Returns The function returns either Sys-1, Sys-1a, Sys-1b, Sys-1c, or Not_Sys_1 string output which indicates
    whether the HVAC system is ASHRAE 90.1 2019 Appendix G system 1 (PTAC), system 1a (system 1 with purchased CHW),
    system 1b (system 1 with purchased heating), system 1c (system 1 with purchased CHW and purchased HW). -------

    """

    is_baseline_system_1_str = HVAC_SYS.UNMATCHED

    if is_baseline_system_1_c(rmd_b, hvac_b_id, terminal_unit_id_list, zone_id_list):
        is_baseline_system_1_str = HVAC_SYS.SYS_1C
    elif is_baseline_system_1_a(rmd_b, hvac_b_id, terminal_unit_id_list, zone_id_list):
        is_baseline_system_1_str = HVAC_SYS.SYS_1A
    else:
        # check if the hvac system has the required sub systems for system type 1
        # if preheat system DOESN'T exist, has_required_sys=True, else, False
        has_required_sys = not has_preheat_system(rmd_b, hvac_b_id)

        are_sys_data_matched = (
            # short-circuit the logic if no required data is found.
            has_required_sys
            # sub functions handles missing required sys, and return False.
            and is_hvac_sys_heating_type_fluid_loop(rmd_b, hvac_b_id)
            and is_hvac_sys_fan_sys_cv(rmd_b, hvac_b_id)
            and does_hvac_system_serve_single_zone(rmd_b, zone_id_list)
            and does_each_zone_have_only_one_terminal(rmd_b, zone_id_list)
            and are_all_terminal_heat_sources_none_or_null(rmd_b, terminal_unit_id_list)
            and are_all_terminal_cool_sources_none_or_null(rmd_b, terminal_unit_id_list)
            and are_all_terminal_fans_null(rmd_b, terminal_unit_id_list)
            and are_all_terminal_types_cav(rmd_b, terminal_unit_id_list)
            and is_hvac_sys_cooling_type_dx(rmd_b, hvac_b_id)
        )
        if are_sys_data_matched:
            if is_hvac_sys_fluid_loop_attached_to_boiler(rmd_b, hvac_b_id):
                is_baseline_system_1_str = HVAC_SYS.SYS_1
            elif is_hvac_sys_fluid_loop_purchased_heating(
                rmd_b, hvac_b_id
            ) and not are_all_terminal_supplies_ducted(rmd_b, terminal_unit_id_list):
                is_baseline_system_1_str = HVAC_SYS.SYS_1B
    return is_baseline_system_1_str
